package sun.awt.image;

import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.image.RasterFormatException;
import java.awt.image.BandedSampleModel;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBuffer;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.image.SampleModel;

public class ByteBandedRaster extends SunWritableRaster
{
    int[] dataOffsets;
    int scanlineStride;
    byte[][] data;
    private int maxX;
    private int maxY;
    
    public ByteBandedRaster(SampleModel sampleModel, Point point) {
        this(sampleModel, sampleModel.createDataBuffer(), new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public ByteBandedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Point point) {
        this(sampleModel, dataBuffer, new Rectangle(point.x, point.y, sampleModel.getWidth(), sampleModel.getHeight()), point, null);
    }
    
    public ByteBandedRaster(SampleModel sampleModel, DataBuffer dataBuffer, Rectangle rectangle, Point point, ByteBandedRaster byteBandedRaster) {
        super(sampleModel, dataBuffer, rectangle, point, byteBandedRaster);
        this.maxX = this.minX + this.width;
        this.maxY = this.minY + this.height;
        if (!(dataBuffer instanceof DataBufferByte)) {
            throw new RasterFormatException("ByteBandedRaster must havebyte DataBuffers");
        }
        DataBufferByte dataBufferByte = (DataBufferByte)dataBuffer;
        if ((sampleModel instanceof BandedSampleModel)) {
            BandedSampleModel bandedSampleModel = (BandedSampleModel)sampleModel;
            this.scanlineStride = bandedSampleModel.getScanlineStride();
            int[] bankIndices = bandedSampleModel.getBankIndices();
            int[] bandOffsets = bandedSampleModel.getBandOffsets();
            int[] offsets = dataBufferByte.getOffsets();
            this.dataOffsets = new int[bankIndices.length];
            this.data = new byte[bankIndices.length][];
            int n = rectangle.x - point.x;
            int n2 = rectangle.y - point.y;
            for (int i = 0; i < bankIndices.length; ++i) {
                this.data[i] = SunWritableRaster.stealData(dataBufferByte, bankIndices[i]);
                this.dataOffsets[i] = offsets[bankIndices[i]] + n + n2 * this.scanlineStride + bandOffsets[i];
            }
            this.verify();
            return;
        }
        throw new RasterFormatException("ByteBandedRasters must haveBandedSampleModels");
    }
    
    public int[] getDataOffsets() {
        return (int[])(int[])this.dataOffsets.clone();
    }
    
    public int getDataOffset(int n) {
        return this.dataOffsets[n];
    }
    
    public int getScanlineStride() {
        return this.scanlineStride;
    }
    
    public int getPixelStride() {
        return 1;
    }
    
    public byte[][] getDataStorage() {
        return this.data;
    }
    
    public byte[] getDataStorage(int n) {
        return this.data[n];
    }
    
    @Override
    public Object getDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array;
        if (o == null) {
            array = new byte[this.numDataElements];
        }
        else {
            array = (byte[])(byte[])o;
        }
        int n3 = (n2 - this.minY) * this.scanlineStride + (n - this.minX);
        for (int i = 0; i < this.numDataElements; ++i) {
            array[i] = this.data[i][this.dataOffsets[i] + n3];
        }
        return array;
    }
    
    @Override
    public Object getDataElements(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array;
        if (o == null) {
            array = new byte[this.numDataElements * n3 * n4];
        }
        else {
            array = (byte[])(byte[])o;
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX);
        for (int i = 0; i < this.numDataElements; ++i) {
            byte[] array2 = this.data[i];
            int n6 = this.dataOffsets[i];
            for (int j = 0; j < n4; ++j, n5 += this.scanlineStride) {
                int n7 = n6 + n5;
                for (int k = 0; k < n3; ++k) {
                    array[i] = array2[n7++];
                    i += this.numDataElements;
                }
            }
        }
        return array;
    }
    
    public byte[] getByteData(int n, int n2, int n3, int n4, int n5, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new byte[this.scanlineStride * n4];
        }
        int n6 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) + this.dataOffsets[n5];
        if (this.scanlineStride == n3) {
            System.arraycopy(this.data[n5], n6, array, 0, n3 * n4);
            return array;
        }
        for (int n7 = 0, i = 0; i < n4; n7 += n3, ++i, n6 += this.scanlineStride) {
            System.arraycopy(this.data[n5], n6, array, n7, n3);
        }
        return array;
    }
    
    public byte[] getByteData(int n, int n2, int n3, int n4, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        if (array == null) {
            array = new byte[this.numDataElements * this.scanlineStride * n4];
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX);
        for (int i = 0; i < this.numDataElements; ++i) {
            byte[] array2 = this.data[i];
            int n6 = this.dataOffsets[i];
            for (int j = 0; j < n4; ++j, n5 += this.scanlineStride) {
                int n7 = n6 + n5;
                for (int k = 0; k < n3; ++k) {
                    array[i] = array2[n7++];
                    i += this.numDataElements;
                }
            }
        }
        return array;
    }
    
    @Override
    public void setDataElements(int n, int n2, Object o) {
        if (n < this.minX || n2 < this.minY || n >= this.maxX || n2 >= this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array = (byte[])(byte[])o;
        int n3 = (n2 - this.minY) * this.scanlineStride + (n - this.minX);
        for (int i = 0; i < this.numDataElements; ++i) {
            this.data[i][this.dataOffsets[i] + n3] = array[i];
        }
        this.markDirty();
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, Raster raster) {
        int n3 = raster.getMinX() + n;
        int n4 = raster.getMinY() + n2;
        int width = raster.getWidth();
        int height = raster.getHeight();
        if (n3 < this.minX || n4 < this.minY || n3 + width > this.maxX || n4 + height > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        this.setDataElements(n3, n4, width, height, raster);
        return;
    }
    
    private void setDataElements(int n, int n2, int w, int n3, Raster raster) {
        if (w <= 0 || n3 <= 0) {
            return;
        }
        int minX = raster.getMinX();
        int minY = raster.getMinY();
        Object dataElements = null;
        for (int i = 0; i < n3; ++i) {
            dataElements = raster.getDataElements(minX, minY + i, w, 1, dataElements);
            this.setDataElements(n, n2 + i, w, 1, dataElements);
        }
        return;
    }
    
    @Override
    public void setDataElements(int n, int n2, int n3, int n4, Object o) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        byte[] array = (byte[])(byte[])o;
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX);
        for (int i = 0; i < this.numDataElements; ++i) {
            byte[] array2 = this.data[i];
            int n6 = this.dataOffsets[i];
            for (int j = 0; j < n4; ++j, n5 += this.scanlineStride) {
                int n7 = n6 + n5;
                for (int k = 0; k < n3; ++k) {
                    array2[n7++] = array[i];
                    i += this.numDataElements;
                }
            }
        }
        this.markDirty();
        return;
    }
    
    public void putByteData(int n, int n2, int n3, int n4, int n5, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n6 = (n2 - this.minY) * this.scanlineStride + (n - this.minX) + this.dataOffsets[n5];
        int n7 = 0;
        if (this.scanlineStride == n3) {
            System.arraycopy(array, 0, this.data[n5], n6, n3 * n4);
        }
        else {
            for (int i = 0; i < n4; n7 += n3, ++i, n6 += this.scanlineStride) {
                System.arraycopy(array, n7, this.data[n5], n6, n3);
            }
        }
        this.markDirty();
        return;
    }
    
    public void putByteData(int n, int n2, int n3, int n4, byte[] array) {
        if (n < this.minX || n2 < this.minY || n + n3 > this.maxX || n2 + n4 > this.maxY) {
            throw new ArrayIndexOutOfBoundsException("Coordinate out of bounds!");
        }
        int n5 = (n2 - this.minY) * this.scanlineStride + (n - this.minX);
        for (int i = 0; i < this.numDataElements; ++i) {
            byte[] array2 = this.data[i];
            int n6 = this.dataOffsets[i];
            for (int j = 0; j < n4; ++j, n5 += this.scanlineStride) {
                int n7 = n6 + n5;
                for (int k = 0; k < n3; ++k) {
                    array2[n7++] = array[i];
                    i += this.numDataElements;
                }
            }
        }
        this.markDirty();
        return;
    }
    
    @Override
    public WritableRaster createWritableChild(int n, int n2, int width, int height, int x, int y, int[] array) {
        if (n < this.minX) {
            throw new RasterFormatException("x lies outside raster");
        }
        if (n2 < this.minY) {
            throw new RasterFormatException("y lies outside raster");
        }
        if (n + width < n || n + width > this.width + this.minX) {
            throw new RasterFormatException("(x + width) is outside raster");
        }
        if (n2 + height < n2 || n2 + height > this.height + this.minY) {
            throw new RasterFormatException("(y + height) is outside raster");
        }
        SampleModel sampleModel;
        if (array != null) {
            sampleModel = this.sampleModel.createSubsetSampleModel(array);
        }
        else {
            sampleModel = this.sampleModel;
        }
        return new ByteBandedRaster(sampleModel, this.dataBuffer, new Rectangle(x, y, width, height), new Point(this.sampleModelTranslateX + (x - n), this.sampleModelTranslateY + (y - n2)), this);
    }
    
    @Override
    public Raster createChild(int n, int n2, int n3, int n4, int n5, int n6, int[] array) {
        return this.createWritableChild(n, n2, n3, n4, n5, n6, array);
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster(int n, int n2) {
        if (n <= 0 || n2 <= 0) {
            throw new RasterFormatException("negative " + ((n <= 0) ? "width" : "height"));
        }
        return new ByteBandedRaster(this.sampleModel.createCompatibleSampleModel(n, n2), new Point(0, 0));
    }
    
    @Override
    public WritableRaster createCompatibleWritableRaster() {
        return this.createCompatibleWritableRaster(this.width, this.height);
    }
    
    private void verify() {
        if (this.width <= 0 || this.height <= 0 || this.height > Integer.MAX_VALUE / this.width) {
            throw new RasterFormatException("Invalid raster dimension");
        }
        if (this.scanlineStride < 0 || this.scanlineStride > Integer.MAX_VALUE / this.height) {
            throw new RasterFormatException("Incorrect scanline stride: " + this.scanlineStride);
        }
        if ((long)this.minX - (long)this.sampleModelTranslateX < 0L || (long)this.minY - (long)this.sampleModelTranslateY < 0L) {
            throw new RasterFormatException("Incorrect origin/translate: (" + this.minX + ", " + this.minY + ") / (" + this.sampleModelTranslateX + ", " + this.sampleModelTranslateY + ")");
        }
        if (this.height > 1 || this.minY - this.sampleModelTranslateY > 0) {
            int i = 0;
            while (i < this.data.length) {
                if (this.scanlineStride > this.data[i].length) {
                    throw new RasterFormatException("Incorrect scanline stride: " + this.scanlineStride);
                }
                ++i;
            }
        }
        int j = 0;
        while (j < this.dataOffsets.length) {
            if (this.dataOffsets[j] < 0) {
                throw new RasterFormatException("Data offsets for band " + j + "(" + this.dataOffsets[j] + ") must be >= 0");
            }
            ++j;
        }
        int n = (this.height - 1) * this.scanlineStride;
        if (this.width - 1 > Integer.MAX_VALUE - n) {
            throw new RasterFormatException("Invalid raster dimension");
        }
        int n2 = n + (this.width - 1);
        int k = 0;
        int l = 0;
        while (l < this.numDataElements) {
            if (this.dataOffsets[l] > Integer.MAX_VALUE - n2) {
                throw new RasterFormatException("Invalid raster dimension");
            }
            int n3 = n2 + this.dataOffsets[l];
            if (n3 > k) {
                k = n3;
            }
            ++l;
        }
        if (this.data.length == 1) {
            if (this.data[0].length <= k * this.numDataElements) {
                throw new RasterFormatException("Data array too small (it is " + this.data[0].length + " and should be > " + k * this.numDataElements + " )");
            }
        }
        else {
            int n4 = 0;
            while (n4 < this.numDataElements) {
                if (this.data[n4].length <= k) {
                    throw new RasterFormatException("Data array too small (it is " + this.data[n4].length + " and should be > " + k + " )");
                }
                ++n4;
            }
        }
        return;
    }
    
    @Override
    public String toString() {
        return new String("ByteBandedRaster: width = " + this.width + " height = " + this.height + " #bands " + this.numDataElements + " minX = " + this.minX + " minY = " + this.minY);
    }
}
